@@ -33,7 +33,7 @@
 
 #include "action.h"
 
-static int action_install_aux(SRCINST_PHANDLE h, char* filename, int reconf);
+static int action_install_aux(SRCINST_PHANDLE h, char* filename, int reconf, int check_conflicts);
 static int action_find_configure(SRCINST_PHANDLE h, char* filename);
 static int action_configure(SRCINST_PHANDLE h);
 static void action_start(void);
@@ -69,7 +69,7 @@ void action_add(char* filename, char* packagename) {
     goto end_proc;
   }
 
-  if (!action_install_aux(h, filename, 1))
+  if (!action_install_aux(h, filename, 1, 1))
     goto end_proc;
 
   result = 1;
@@ -88,9 +88,11 @@ void action_add(char* filename, char* packagename) {
   }
 }
 
-static int action_install_aux(SRCINST_PHANDLE h, char* filename, int reconf) {
+static int action_install_aux(SRCINST_PHANDLE h, char* filename, int reconf, int check_conflicts) {
   float v;			/* autoconf version */
   SRCINST_ERR rv;
+  SRCINST_MFEATS mf;
+
   char* title = "Installation";
 
   status_done(0.0, 10.0);
@@ -126,16 +128,84 @@ static int action_install_aux(SRCINST_PHANDLE h, char* filename, int reconf) {
     }
   }
   
-  status_done(20.0, 98.0);
+  status_done(20.0, 25.0);
 
-  if ((rv = srcinst_install(h, state.o.install, state.o.source, state.o.strip,
-			    0, state.o.src_compress)) != SRCINST_ERR_OK) {
-    action_error(rv, title, "Installation failed", "performing actual installation.");
+  if (!(mf = srcinst_detect_makefile_features(h))) {
+    dialog_warning(title, "Makefile detection failed.", "Failed to detect Makefile or Makefile features.", 0);
+    return 0;
+  }
+  
+  if (!srcinst_is_feature_supported(mf, SRCINST_FEATURE_DESTDIR) &&
+      !srcinst_is_feature_supported(mf, SRCINST_FEATURE_INSTALL_ROOT)) {
+    dialog_warning(title, "Staged installation not supported", "This package does not support required staged installation features.", 0);
+    srcinst_free_makefile_features(mf);
     return 0;
   }
 
-  status_done(98.0, 100.0);
+  if (state.o.source) {
+    if ((rv = srcinst_install_source(h, mf, state.o.src_compress)) != SRCINST_ERR_OK) {
+      action_error(rv, title, "Source installation failed", "installing the configured source code for later use. Check console for the reasons behind the error.");
+      srcinst_free_makefile_features(mf);
+      return 0;
+    }
+  }
+  
+  status_done(25.0, 90.0);
+  
+  if ((rv = srcinst_build(h, mf)) != SRCINST_ERR_OK) {
+    action_error(rv, title, "Build failed", "compiling the source code. Check if your system is supported by the package you want to install, and check configuration options. It could also be a bug in the package itself. You can generate a bug report quickly by right-clicking in the console window, and choosing Save Log.");
+    srcinst_free_makefile_features(mf); return 0;
+  }
+  
+  if (state.o.install) {
+    status_done(90.0, 92.5);
 
+    if ((rv = srcinst_test_install(h, mf, state.o.strip)) != SRCINST_ERR_OK) {
+      action_error(rv, "Test installation failed", "performing the test installation. Therefore, real installation will not be attempted.", 0);
+      srcinst_free_makefile_features(mf); return 0;
+    }
+
+    if (check_conflicts) {
+      struct srcinst_string_list conflicts;
+      
+      if (!srcinst_get_conflict_list(h, &conflicts)) {
+	dialog_warning(title, "Conflict test failed", "An unexpected error has occured trying to perform the package conflict test", 0);
+	srcinst_free_makefile_features(mf); return 0;
+      }
+
+      if (conflicts.count > 0) {
+	int i; char* conflicts_string = 0;
+
+	for (i = 0; i < conflicts.count; i++) {
+	  conflicts_string = srcinst_append(conflicts_string, "\n");
+	  conflicts_string = srcinst_append(conflicts_string,
+					    conflicts.strings[i]);
+	}
+
+	conflicts_string = srcinst_append(conflicts_string, "\n\n  Do you really want to overwrite the already claimed files?");
+	i = dialog_confirm(title, "Conflicts detected", "Installing this package would overwrite files which other packages claim. The files involved are:\n", conflicts_string);
+	free(conflicts_string);
+
+	if (!i) {
+	  srcinst_free_string_list(&conflicts);
+	  srcinst_free_makefile_features(mf);
+	  return 0;
+	}
+      }
+      srcinst_free_string_list(&conflicts);
+    }
+
+    status_done(92.5, 100.0);
+    
+    if ((rv = srcinst_install_binary(h, mf, state.o.strip)) != SRCINST_ERR_OK) {
+      action_error(rv, title, "Binary installation failed", "installing binaries in their final location. See the console for the reasons behind the error. Remember to check your permissions on the chosen installation prefix.");
+      srcinst_free_makefile_features(mf); return 0;
+    }
+  }
+  
+  status_done(99.0, 100.0);
+  
+  srcinst_free_makefile_features(mf);
   return 1;
 }
 
@@ -147,7 +217,8 @@ static int action_configure(SRCINST_PHANDLE h) {
   char* title = "Configuration";
   
   srcinst_init_string_list(&configure_strings);
-  
+  srcinst_add_string_list(&configure_strings, "./configure");
+
   if (state.o.manual_conf) {
     if ((rv = srcinst_detect_configure_opt(h, &sets)) != SRCINST_ERR_OK) {
       action_error(rv, title, "Failed to detect configure options", "trying to detect supported configure options.");
@@ -236,7 +307,7 @@ void action_install(int reconf) {
     goto end_proc;
   }
   
-  if (!action_install_aux(state.package, source, reconf))
+  if (!action_install_aux(state.package, source, reconf, 1))
     goto end_proc;
 
   result = 1;
@@ -265,17 +336,14 @@ void action_remove(void) {
   status_done(0.0, 10.0);
   
   if (srcinst_is_installed(state.package)) {
-    if (srcinst_is_source_available(state.package)) {
+    if (srcinst_is_source_available(state.package) &&
+	srcinst_get_installed_files(state.package)->count == 0) {
       rv = srcinst_init_build(state.package,
 			      srcinst_get_source_location(state.package), 0);
-
+      
       if (rv != SRCINST_ERR_OK) {
-	action_error(rv, title, "Could not initialize build", "initializing source build for optional source-assisted uninstallation.");
-	
-	if (rv == SRCINST_ERR_INTR)
-	  goto end_proc;
-	/* ignore other errors of init_build, because we can uninstall
-	   internally if source is not available/does not work */
+	action_error(rv, title, "Could not initialize build", "initializing source build for source-assisted uninstallation.");
+	goto end_proc;
       }
     }
 
@@ -322,17 +390,15 @@ void action_uninstall(void) {
 
   status_done(0.0, 10.0);
 
-  if (srcinst_is_source_available(state.package)) {
+  if (srcinst_is_source_available(state.package) && 
+      srcinst_get_installed_files(state.package)->count == 0) {
     rv = srcinst_init_build(state.package, 
 			    srcinst_get_source_location(state.package), 0);
 
     if (rv != SRCINST_ERR_OK) {
-      action_error(rv, title, "Could not initialize build", "initializing source build for optional source-assisted uninstallation.");
-	
-      if (rv == SRCINST_ERR_INTR)
-	goto end_proc;
-      /* ignore other errors of init_build, because we can uninstall
-	 internally if source is not available/does not work */
+      action_error(rv, title, "Could not initialize build", "initializing source build for source-assisted uninstallation.");
+      
+      goto end_proc;
     }
   }
 
@@ -384,7 +450,7 @@ void action_upgrade(char* filename, char* packagename) {
     goto end_proc;
   }
 
-  if (!action_install_aux(h, filename, 1))
+  if (!action_install_aux(h, filename, 1, 0))
     goto end_proc;
 
   if ((descr = srcinst_get_descr(state.package))) {
@@ -398,22 +464,14 @@ void action_upgrade(char* filename, char* packagename) {
   result = 1;
   
  end_proc:
-  if (h && !srcinst_close_package(h)) {
-    dialog_warning(title, "Error updating new package information", "An error condition has been detected trying to update and close the package information.", 0);
-    result = 0;
-  }
 
   if (result) {
-    if (srcinst_upgrade_package(state.package) != SRCINST_ERR_OK)
-      result = 0;
-    if (!srcinst_close_package(state.package))
-      result = 0;
-    
-    if (!result) {
-      dialog_warning(title, "Error updating old package information", "An error condition has been detected trying to close the old package information.", 0);
+    SRCINST_ERR rv;
+    if ((rv = srcinst_upgrade_package(state.package, h)) != SRCINST_ERR_OK) {
+      action_error(rv, title, "Error upgrading package information.", "trying to upgrade package information.");
     }
   }
-
+  
   status_done(100.0, 100.0);
   action_end();
 
