@@ -29,6 +29,7 @@
 #include "edit.h"
 #include "pref.h"
 #include "events.h"
+#include "console.h"
 
 /* note that not all event bindings are set here.
    Widgets built dynamically are connected when created.
@@ -64,6 +65,11 @@ void events_bind(void) {
   g_signal_connect(G_OBJECT(w_edit[W_EDIT_CANCEL]), "clicked",
 		   G_CALLBACK(edit_cancel), NULL);
 
+  /* console */
+
+  g_signal_connect(G_OBJECT(w_console[W_CONSOLE_TXT]), "populate-popup",
+		   G_CALLBACK(console_populate_popup), NULL);
+
   /* pref */
 
   g_signal_connect(G_OBJECT(w_pref[W_PREF]), "delete-event",
