@@ -42,10 +42,57 @@
 GtkWidget* w_main[W_MAIN_N];
 GtkAccelGroup* acc;
 
+void gui_splash(void) {
+  GtkWidget* splash;
+  w_main[W_SPLASH] = gtk_window_new(GTK_WINDOW_POPUP);
+  splash = gtk_image_new_from_file(DATADIR "/images/splash.png");
+
+  gtk_window_set_decorated(GTK_WINDOW(w_main[W_SPLASH]), FALSE);
+  gtk_container_add(GTK_CONTAINER(w_main[W_SPLASH]), splash);
+  gtk_window_set_position(GTK_WINDOW(w_main[W_SPLASH]),
+			  GTK_WIN_POS_CENTER_ALWAYS);
+
+  gtk_widget_show_all(w_main[W_SPLASH]);
+  gui_doevents(0);
+}
+
+
+void gui_unsplash(void) {
+  gtk_widget_destroy(w_main[W_SPLASH]);
+}
+
+static void gui_icon_init(void) {
+  /*
+  GdkPixbuf* sizes[5] = { 0, 0, 0, 0, 0 };
+  char* filenames[5] = { DATADIR "/images/icon16x16.png",
+			 DATADIR "/images/icon24x24.png",
+			 DATADIR "/images/icon32x32.png",
+			 DATADIR "/images/icon48x48.png",
+			 0 };
+  int i;
+  GList* icon_list = NULL;
+
+  for (i = 0; filenames[i]; i++) {
+    sizes[i] = gdk_pixbuf_new_from_file(filenames[i], NULL);
+    if (sizes[i])
+      icon_list = g_list_append(icon_list, sizes[i]);
+  }
+  
+  if (icon_list)
+    gtk_window_set_default_icon_list(icon_list);
+  */
+
+  GdkPixbuf* buf;
+  if ((buf = gdk_pixbuf_new_from_file(DATADIR "/images/icon48x48.png", NULL)))
+    gtk_window_set_default_icon(buf);
+}
+
 void gui_init(void) {
   GtkWidget* left, *right;
   printf("gui_init\n");
 
+  gui_icon_init();
+
   text_init();			/* initialize text tags */
   w_main[W_MAIN] = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title(GTK_WINDOW(w_main[W_MAIN]), "GNU Source Installer");
@@ -269,6 +316,37 @@ void gui_action_export_bin(GtkWidget* widget, gpointer data) {
     action_export_bin(filename, state.o.src_compress);
 }
 
+void gui_action_savelog(GtkWidget* w, gpointer unused_data) {
+  char* filename = 0; char* title = "Save console log";
+  GtkWidget* dialog; gint response;
+  printf("gui_action_savelog\n");
+
+  dialog = gtk_file_chooser_dialog_new(title, GTK_WINDOW(w_main[W_MAIN]),
+				       GTK_FILE_CHOOSER_ACTION_SAVE,
+				       GTK_STOCK_SAVE, GTK_RESPONSE_OK,
+				       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				       NULL);
+
+  gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog), "console.log");
+
+  gtk_widget_show_all(dialog);
+  response = gtk_dialog_run(GTK_DIALOG(dialog));
+
+  if (response == GTK_RESPONSE_OK)
+    filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
+
+  gtk_widget_destroy(dialog);
+  
+  if (filename) {
+    if (console_savelog(filename)) {
+      dialog_message(title, "Console log saved.", "The console log has been saved as", filename);
+    } else {
+      dialog_warning(title, "Error saving console log", "Console log save could NOT be completed successfully", 0);
+    }
+    free(filename);
+  }
+}
+
 void gui_action_default_size(GtkWidget* unused_w, gpointer unused_d) {
   printf("gui_action_default_size\n");
   gui_set_default_size();
